@@ -1,15 +1,19 @@
 extern crate tesseract_sys;
 extern crate thiserror;
 
+#[cfg(feature = "tesseract_5_2")]
+use self::tesseract_sys::TessBaseAPIInit5;
 use self::tesseract_sys::{
     TessBaseAPIAllWordConfidences, TessBaseAPICreate, TessBaseAPIDelete, TessBaseAPIGetAltoText,
     TessBaseAPIGetComponentImages, TessBaseAPIGetHOCRText, TessBaseAPIGetInputImage,
     TessBaseAPIGetLSTMBoxText, TessBaseAPIGetSourceYResolution, TessBaseAPIGetTsvText,
     TessBaseAPIGetUTF8Text, TessBaseAPIGetWordStrBoxText, TessBaseAPIInit2, TessBaseAPIInit3,
     TessBaseAPIMeanTextConf, TessBaseAPIRecognize, TessBaseAPISetImage, TessBaseAPISetImage2,
-    TessBaseAPISetRectangle, TessBaseAPISetSourceResolution, TessBaseAPISetVariable,
-    TessDeleteIntArray, TessOcrEngineMode, TessPageIteratorLevel,
+    TessBaseAPISetPageSegMode, TessBaseAPISetRectangle, TessBaseAPISetSourceResolution,
+    TessBaseAPISetVariable, TessDeleteIntArray, TessOcrEngineMode, TessPageIteratorLevel,
+    TessPageSegMode,
 };
+
 use self::thiserror::Error;
 use crate::Text;
 use leptonica_plumbing::Pix;
@@ -131,6 +135,36 @@ impl TessBaseApi {
         Self(unsafe { TessBaseAPICreate() })
     }
 
+    #[cfg(feature = "tesseract_5_2")]
+    /// Wrapper for [`Init-1`]https://tesseract-ocr.github.io/tessapi/5.x/a02438.html#a2be07b4c9449b8cfc43e9c26ee623050
+    pub fn init_1(
+        &mut self,
+        data: &[u8],
+        language: Option<&CStr>,
+        oem: TessOcrEngineMode,
+    ) -> Result<(), TessBaseApiInitError> {
+        let ret = unsafe {
+            TessBaseAPIInit5(
+                self.0,
+                data.as_ptr().cast(),
+                data.len() as c_int,
+                language.map(CStr::as_ptr).unwrap_or_else(ptr::null),
+                oem,
+                ptr::null_mut(),
+                0,
+                ptr::null_mut(),
+                ptr::null_mut(),
+                0,
+                0,
+            )
+        };
+        if ret == 0 {
+            Ok(())
+        } else {
+            Err(TessBaseApiInitError {})
+        }
+    }
+
     /// Wrapper for [`Init-2`](https://tesseract-ocr.github.io/tessapi/5.x/a02438.html#a965ef2ff51c440756519a3d6f755f34f)
     ///
     /// Start tesseract
@@ -244,6 +278,14 @@ impl TessBaseApi {
             _ => Err(TessBaseApiSetVariableError {}),
         }
     }
+
+    /// Wrapper for [`SetPageSegMode`](https://tesseract-ocr.github.io/tessapi/5.x/a02438.html#a15a7a9c1afbba3078a55b4566de891ab)
+    ///
+    /// Set the current page segmentation mode
+    pub fn set_page_seg_mode(&mut self, mode: TessPageSegMode) {
+        unsafe { TessBaseAPISetPageSegMode(self.0, mode) };
+    }
+
     /// Wrapper for [`Recognize`](https://tesseract-ocr.github.io/tessapi/5.x/a02438.html#a0e4065c20b142d69a2324ee0c74ae0b0)
     ///
     /// Recognize the image. Returns `Ok(())` on success and `Err(())` otherwise.
@@ -290,12 +332,18 @@ impl TessBaseApi {
     }
 
     /// Wrapper for [`TessBaseAPIGetInputImage`](https://tesseract-ocr.github.io/tessapi/5.x/a00008.html#ad2c023e46bf634305b3ae8cd0c091a65)
-    pub fn get_input_image(&self) -> Option<leptonica_plumbing::BorrowedPix> {
+    pub fn get_input_image(
+        &self,
+    ) -> Option<leptonica_plumbing::memory::BorrowedFrom<leptonica_plumbing::Pix>> {
         let ptr = unsafe { TessBaseAPIGetInputImage(self.0) };
         if ptr.is_null() {
             None
         } else {
-            Some(unsafe { leptonica_plumbing::BorrowedPix::new(ptr) })
+            Some(unsafe {
+                leptonica_plumbing::memory::BorrowedFrom::new(
+                    leptonica_plumbing::Pix::new_from_pointer(ptr),
+                )
+            })
         }
     }
 
@@ -407,7 +455,10 @@ impl TessBaseApi {
         &self,
         level: TessPageIteratorLevel,
         text_only: c_int,
-    ) -> Result<leptonica_plumbing::Boxa, TessBaseApiGetComponentImagesError> {
+    ) -> Result<
+        leptonica_plumbing::memory::RefCountedExclusive<leptonica_plumbing::Boxa>,
+        TessBaseApiGetComponentImagesError,
+    > {
         let ptr = unsafe {
             TessBaseAPIGetComponentImages(
                 self.0,
@@ -420,7 +471,11 @@ impl TessBaseApi {
         if ptr.is_null() {
             Err(TessBaseApiGetComponentImagesError {})
         } else {
-            Ok(unsafe { leptonica_plumbing::Boxa::new_from_pointer(ptr) })
+            Ok(unsafe {
+                leptonica_plumbing::memory::RefCountedExclusive::new(
+                    leptonica_plumbing::Boxa::new_from_pointer(ptr),
+                )
+            })
         }
     }
 }
